<?php
require __DIR__ . "/config.php";

if ($argv[1] != 1) {
    if (time() < $nextRun)
        exit;
}
file_put_contents("test.txt", 'ran at ' . date("Y-m-d H:i:s"));

$uploaderUrlList = explode("\r\n", $config->get("uploaderUrlList"));

$cron = \Cron\CronExpression::factory($config->get("cronExpression"));
$nextRun = $cron->getNextRunDate()->getTimestamp();
file_put_contents($nextRunPath, $nextRun);

$handler = new \Monolog\Handler\StreamHandler(__DIR__ . "/cron-logs.txt");

$logger->pushHandler($handler);

$scraper = new \Scraper\NatuerlichScraper($client, $logger);
$logsRepository->insert(new \Scraper\Log(false, "Scraping started..."));
try {
    $productList = $scraper->scrape();
    $logsRepository->insert(new \Scraper\Log(false, "Found " . count($productList) . " products"));
    file_put_contents($productListPath, serialize($productList));
//    $productList = unserialize(file_get_contents($productListPath));

    $imageScraper = new \Scraper\ImageScraper($client, $logger, $imageDirectoryPath, $tmpDirectoryPath);
    foreach ($productList as $product) {
        try {
            if ($product->imageUrl != null) {
                $logger->debug("Fetching Image");
                if ($imageScraper->shouldScrape($product)) {
                    $imageScraper->scrapeProductImage($product);
                }
            }
        } catch (Exception $e) {
            $logger->error("Can not scrape product image! Id: " . $product->id . ". Message: " . $e->getMessage());
        }
    }
}catch (\Exception $e){
    $logsRepository->insert(new \Scraper\Log(true, "Scraping problem... " . $e->getMessage()));
}

$defaultHeaders = [
    "Accept" => "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8",
    "Accept-Encoding" => "gzip, deflate, sdch, br",
    "Accept-Language" => "pl-PL,pl;q=0.8,en-US;q=0.6,en;q=0.4",
    "Cache-Control" => "max-age=0",
    "Connection" => "keep-alive",
//    "Host" => "www.birki.co",
    "Upgrade-Insecure-Requests" => "1",
    "User-Agent" => "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/55.0.2883.87 Safari/537.36",
];

$client = new \GuzzleHttp\Client([
    "defaults" => [
        "verify" => false,
        "headers" => $defaultHeaders
    ]
]);
foreach ($uploaderUrlList as $uploaderUrl) {
    $logger->info("Running uploader: $uploaderUrl");
    try {
        $result = $client->get($uploaderUrl);
        $body = $result->getBody()->getContents();
        $logger->info("Response: $body");
        if($body !== "OK!"){
            throw new \Exception($body);
        }else{
            $logsRepository->insert(new \Scraper\Log(false, "Uploader $uploaderUrl ended successfully"));
            $logger->info("Uploader ended successfully!");
        }
    }catch (\Exception $e){
        $logger->err("Can not connect to $uploaderUrl. " . $e->getMessage());
        $logsRepository->insert(new \Scraper\Log(true, "Can not run uploader $uploaderUrl. Error: " . $e->getMessage()));
    }
}