<?php
/**
 * Created by PhpStorm.
 * User: Jaskolek
 * Date: 2016-12-14
 * Time: 13:01
 */

namespace Scraper;


use GuzzleHttp\Client;
use Monolog\Logger;

class ImageScraper
{
    const PART_WIDTH = 256;
    const PART_HEIGHT = 256;

    /**
     * @var Client
     */
    protected $client;

    /**
     * @var Logger
     */
    protected $logger;


    protected $imageDirectoryPath;
    protected $tmpDirectoryPath;

    /**
     * ImageScraper constructor.
     * @param Client $client
     * @param Logger $logger
     * @param $imageDirectoryPath
     * @param $tmpDirectoryPath
     */
    public function __construct(Client $client, Logger $logger, $imageDirectoryPath, $tmpDirectoryPath)
    {
        $this->client = $client;
        $this->logger = $logger;
        $this->imageDirectoryPath = $imageDirectoryPath;
        $this->tmpDirectoryPath = $tmpDirectoryPath;
    }


    public function scrapeProductImage(Product $product)
    {

        $parts = explode(".", $product->imageUrl);
        $extension = strtolower(array_pop($parts));

        $this->logger->debug("Fetching image for id: $product->id");
        $outPath = $this->getProductOutPath($product);
        $tmpPath = $this->tmpDirectoryPath . "/" . $product->id . "." . $extension;

        $this->logger->debug("Fetching url: " . $product->imageUrl);
        $this->client->get($product->imageUrl, [
            "save_to" => $tmpPath
        ]);

        if($extension == "jpg" || $extension == "jpeg"){
            $im = imagecreatefromjpeg($tmpPath);
        }else if($extension == "gif"){
            $im = imagecreatefromgif($tmpPath);
        }else if($extension == "png"){
            $im = imagecreatefrompng($tmpPath);
        }else{
            throw new \Exception("Unknow format $extension!");
        }
        imagejpeg($im, $outPath);
        unlink($tmpPath);
        return $outPath;
    }

    public function shouldScrape(Product $product)
    {
        $outPath = $this->getProductOutPath($product);

        $this->logger->debug("should Scrape $product->id?");
        if (!file_exists($outPath)) return true;

        $url = $product->imageUrl;
        $lastModified = strtotime($this->client->get($url)->getHeader("Last-Modified"));
        $localFileLastModified = filemtime($outPath);

        return $localFileLastModified < $lastModified;
    }


    protected function getProductOutPath(Product $product)
    {
        return $this->imageDirectoryPath . "/" . $product->id . ".jpg";
    }
}