<?php
/**
 * Created by PhpStorm.
 * User: jasko
 * Date: 19.01.2017
 * Time: 16:08
 */

namespace Scraper;

class LogsRepository
{
    /**
     * @var \PDO
     */
    protected $pdo;

    protected $tableName;



    public function __construct(\PDO $pdo, $tableName = "logs")
    {
        $this->pdo = $pdo;
        $this->tableName = $tableName;

        $sql = "CREATE TABLE IF NOT EXISTS " . $this->tableName . "("
            . "id INTEGER PRIMARY KEY,"
            . "isError TINYINT(1) NOT NULL,"
            . "message TEXT,"
            . "insertTimestamp INT(12)"
            . ");";

        $this->pdo->exec($sql);
    }

    /**
     * @return \Scraper\Log[]
     */
    public function fetchAll()
    {
        $stmt = $this->pdo->prepare("SELECT * FROM " . $this->tableName . " ORDER BY insertTimestamp DESC LIMIT 50");
        $stmt->execute();
        $rows = $stmt->fetchAll();

        return array_map(function($row){
            return $this->arrayToLog($row);
        }, $rows);
    }


    public function insert(Log $log)
    {
        $row = $this->logToArray($log);

        $stmt = $this->pdo->prepare("INSERT INTO " . $this->tableName
            . "('id', 'isError', 'message', 'insertTimestamp')"
            . "VALUES"
            . "(:id, :isError, :message, :insertTimestamp)"
        );
        $stmt->execute($row);
    }

    public function insertMany($orderResultList)
    {
        foreach ($orderResultList as $orderResult) {
            $this->insert($orderResult);
        }
    }

    protected function logToArray(Log $log)
    {
        $row = [
            "id" => $log->id,
            "isError" => $log->isError,
            "message" => $log->message,
            "insertTimestamp" => $log->insertTimestamp
        ];

        return $row;
    }

    /**
     * @param $array
     * @return Log
     */
    protected function arrayToLog($array)
    {
        $log = new Log(
            $array["isError"],
            $array["message"],
            $array["insertTimestamp"]
        );

        return $log;
    }

}