<?php
namespace Ordering;


use GuzzleHttp\Client;
use Symfony\Component\DomCrawler\Crawler;

class JoomlaApi
{
    /**
     * @var Client
     */
    protected $client;

    public function updateOrder(JoomlaOrder $order, $status = "preparing to dispatch")
    {
        $body = $this->client->get("/administrator/index.php?option=com_hikashop&ctrl=order&task=changestatus&order_id=" . $order->id . "&tmpl=component&status=" . $status);

        $crawler = new Crawler();
        $crawler->addHtmlContent($body);

        $inputList = $this->getInputList($crawler);
        $inputList["data[order][history][history_reason]"] = "";
        $inputList["data[order][history][history_notified]"] = 1;
        $inputList["task"] = "savechangestatus";

        $this->client->post("/administrator/index.php?option=com_hikashop&ctrl=order&tmpl=component", [
            "body" => $inputList
        ]);
    }

    /**
     * @return JoomlaOrder[]
     */
    public function fetchOrderList()
    {
        $body = $this->client->get("/administrator/index.php?option=com_hikashop&ctrl=order");

        $crawler = new Crawler();
        $crawler->addHtmlContent($body);

        $inputList = $this->getInputList($crawler);
//        $inputList["filter_status"] = [""];
        $inputList["filter_status"] = ["confirmed"];
        $inputList["limit"] = 500;

        $body = $this->client->post("/administrator/index.php?option=com_hikashop&ctrl=order", [
            "body" => $inputList,
        ])->getBody()->getContents();

        $crawler = new Crawler();
        $crawler->addHtmlContent($body);

        $urlList = $crawler->filter(".hikashop_order_number_value a")->each(function(Crawler $crawler){
            return $crawler->attr("href");
        });

        $orderList = [];
        foreach($urlList as $url){
            $orderList[] = $this->fetchOrderData($url);
        }

        return $orderList;
    }

    /**
     * @param $url
     * @return JoomlaOrder
     */
    public function fetchOrderData($url)
    {
        $body = $this->client->get($url);
        $order = new JoomlaOrder();
        $crawler = new Crawler();
        $crawler->addHtmlContent($body);

        $order->id = trim($crawler->filter(".hikashop_order_id td")->last()->text());
        $order->orderNumber = trim($crawler->filter(".hikashop_order_ordernumber td")->last()->text());

        $order->joomlaOrderProductList = $crawler->filter("#hikashop_order_product_listing tbody tr")->each(function(Crawler $crawler){
            $name = $crawler->filter(".hikashop_order_item_name_value")->html();
            preg_match("|([0-9]+-[0-9]{2})|", $name, $matches);
            $productCode = $matches[1];
            $quantity = $crawler->filter(".hikashop_order_item_quantity_value")->text();

            $orderProduct = new JoomlaOrderProduct($productCode, $quantity);
            return $orderProduct;
        });

        return $order;
    }

    
    /**
     * JoomlaApi constructor.
     * @param Client $client
     */
    public function __construct(Client $client)
    {
        $this->client = $client;
    }

    public function logIn($username, $password)
    {
        $body = $this->client->get("/administrator/")->getBody()->getContents();
        $crawler = new Crawler();
        $crawler->addHtmlContent($body);

        $inputList = [];
        $crawler->filter("form input")->each(function (Crawler $crawler) use (&$inputList) {
            $key = $crawler->attr("name");
            $value = $crawler->attr("value");
            $inputList[$key] = $value;
        });


        $inputList["username"] = $username;
        $inputList["passwd"] = $password;
        $inputList["lang"] = "";

        $body = $this->client->post("/administrator/index.php", [
            "body" => $inputList,
        ])->getBody()->getContents();

        $crawler = new Crawler();
        $crawler->addHtmlContent($body);
        if (!$this->isLoggedIn()) {
            throw new \Exception("Can not log in!");
        }
    }
    protected function getInputList(Crawler $crawler)
    {
        $inputList = [];
        $crawler->filter("form input")->each(function (Crawler $crawler) use (&$inputList) {
            $key = $crawler->attr("name");
            $value = $crawler->attr("value");
            $inputList[$key] = $value;
        });

        $crawler->filter("form textarea")->each(function (Crawler $crawler) use (&$inputList) {
            $key = $crawler->attr("name");
            $value = $crawler->html();
            $inputList[$key] = $value;
        });

        return $inputList;
    }

    public function isLoggedIn()
    {
        $body = $this->client->get("/administrator/")->getBody()->getContents();
        $crawler = new Crawler();
        $crawler->addHtmlContent($body);
        return $crawler->filter("body.admin")->count() == 1;
    }
}