<?php
/**
 * Created by PhpStorm.
 * User: jasko
 * Date: 13.01.2017
 * Time: 01:13
 */

namespace Ordering;


use GuzzleHttp\Client;
use Symfony\Component\DomCrawler\Crawler;

class NatuerlichApi
{
    /**
     * @var Client
     */
    protected $client;

    protected $token = null;

    /**
     * NatuerlichApi constructor.
     * @param Client $client
     */
    public function __construct(Client $client)
    {
        $this->client = $client;
    }


    public function isLoggedIn()
    {
        $body = $this->client->get("/account")->getBody()->getContents();

        $crawler = new Crawler();
        $crawler->addHtmlContent($body);

        return $crawler->filter("form[name='sLogin']")->count() == 0;
    }

    public function logIn($username, $password)
    {

        $body = $this->client->get("/account")->getBody()->getContents();

        $crawler = new Crawler();
        $crawler->addHtmlContent($body);

        $formFields = [
            "sTarget" => "account",
            "email" => $username,
            "password" => $password,
            "Submit" => "",
            "__csrf_token" => $this->getCsrfToken()
        ];

        $body = $this->client->post("/account/login/sTarget/account/sTargetAction/index", [
            "body" => $formFields
        ])->getBody()->getContents();

    }

    public function getCsrfToken()
    {
        if ($this->token === null) {
            $this->token = $this->client->get("/csrftoken")->getHeader("X-Csrf-Token");
        }
        return $this->token;
    }

    public function clearCart()
    {
        $body = $this->client->get("/checkout/cart")->getBody()->getContents();

        $crawler = new Crawler();
        $crawler->addHtmlContent($body);

        $deleteUrlList = $crawler->filter(".row--product .column--actions form")->each(function (Crawler $crawler) {
            return $crawler->attr("action");
        });

        foreach ($deleteUrlList as $deleteUrl) {
            $this->client->post($deleteUrl, [
                "body" => [
                    "__csrf_token" => $this->getCsrfToken()
                ]
            ]);
        }


        $body = $this->client->get("/checkout/cart")->getBody()->getContents();

        $crawler = new Crawler();
        $crawler->addHtmlContent($body);

        $success = ($crawler->filter(".basket--info-messages .is--warning")->count() != 0);

        if (!$success) {
            throw new \Exception("Can not clear cart!");
        }
    }

    public function getCartPrice()
    {
        $body = $this->client->get("/checkout/confirm")->getBody()->getContents();
        $crawler = new Crawler();
        $crawler->addHtmlContent($body);

        $priceText = $crawler->filter(".entry--sum")->text();

        $priceText = preg_replace("|[^0-9,]|", "", $priceText);
        $price = (float)str_replace(",", ".", $priceText);
        return $price;
    }

    public function makeOrder()
    {
        $body = $this->client->get("/checkout/confirm")->getBody()->getContents();

        $postFields = [
            "sComment" => "",
            "sAGB" => "on",
            "__csrf_token" => $this->getCsrfToken()
        ];

        $body = $this->client->post("/checkout/finish", [
            "body" => $postFields
        ])->getBody()->getContents();


        $crawler = new Crawler();
        $crawler->addHtmlContent($body);

        if($crawler->filter(".is--error")->count() > 0){
            $error = $crawler->filter(".is--error")->text();
            throw new \Exception($error);
        }

        if($crawler->filter(".teaser--title")->count() > 0){
            //done withour any errors!
        }else{
            throw new \Exception("Unknown error!");
        }
    }

    public function getCart()
    {
        return $this->client->get("/checkout/cart")->getBody()->getContents();
    }

    public function orderJoomlaOrderProduct(JoomlaOrderProduct $joomlaOrderProduct)
    {
        $url = $this->findProductUrlFromJoomlaOrderProduct($joomlaOrderProduct->id);
        $this->client->get($url);

        $id = $this->getProductId($joomlaOrderProduct->id);
        $addUrl = "/checkout/ajaxAddArticleCart?sAdd=$id&sQuantity=" . $joomlaOrderProduct->quantity;

        $body = $this->client->get($addUrl);
        $crawler = new Crawler();
        $crawler->addHtmlContent($body);

        $isSuccess = $crawler->filter(".is--success")->count() > 0;
        if (!$isSuccess) {
            $message = $crawler->filter(".alert--content")->text();
            $message = $joomlaOrderProduct->id . " - " . $url . " - " . $message;
            throw new \Exception($message);
        }
    }

    public function findProductUrlFromJoomlaOrderProduct($joomlaOrderProductCode)
    {
        $this->getCsrfToken();
        $id = $this->getProductId($joomlaOrderProductCode);
        $url = "/search?sSearch=$id";
        $response = $this->client->get($url, [
            "allow_redirects" => false
        ]);

        $status = $response->getStatusCode();

        if ($status != 302) {
            throw new \Exception("Can not find product for $joomlaOrderProductCode");
        }

        $url = $response->getHeader("location");
        return $url;
    }

    protected function getProductId($joomlaOrderProductCode)
    {
        $id = str_replace("-", "", $joomlaOrderProductCode);
        $id = $id . "0";
        $id = str_pad($id, 11, '0', STR_PAD_LEFT);

        return $id;
    }
}