<?php
/**
 * Created by PhpStorm.
 * User: jasko
 * Date: 13.01.2017
 * Time: 15:17
 */

namespace Ordering;


class OrderResultRepository
{
    /**
     * @var \PDO
     */
    protected $pdo;

    protected $tableName;



    public function __construct(\PDO $pdo, $tableName = "orderResult")
    {
        $this->pdo = $pdo;
        $this->tableName = $tableName;

        $sql = "CREATE TABLE IF NOT EXISTS " . $this->tableName . "("
            . "id VARCHAR(255) PRIMARY KEY,"
            . "success TINYINT(1) NOT NULL,"
            . "message TEXT,"
            . "insertTimestamp INT(12),"
            . "orderData TEXT NOT NULL"
            . ");";

        $this->pdo->exec($sql);
    }

    /**
     * @return OrderResult[]
     */
    public function fetchAll()
    {
        $stmt = $this->pdo->prepare("SELECT * FROM " . $this->tableName);
        $stmt->execute();
        $rows = $stmt->fetchAll();

        return array_map(function($row){
            return $this->arrayToOrderResult($row);
        }, $rows);
    }

    /**
     * @param $time
     * @return OrderResult[]
     */
    public function fetchYoungerThan($time)
    {
        $stmt = $this->pdo->prepare("SELECT * FROM " . $this->tableName . " WHERE insertTimestamp > $time");
        $stmt->execute();
        $rows = $stmt->fetchAll();

        return array_map(function($row){
            return $this->arrayToOrderResult($row);
        }, $rows);
    }

    public function delete($id)
    {
        $stmt = $this->pdo->prepare("DELETE FROM " . $this->tableName . " WHERE id = :id");
        $stmt->execute(["id" => $id]);
    }

    public function findBySuccess($success)
    {
        $stmt = $this->pdo->prepare("SELECT * FROM " . $this->tableName . " WHERE success = :success");
        $stmt->execute(["success" => $success]);
        $rows = $stmt->fetchAll();

        return array_map(function($row){
            return $this->arrayToOrderResult($row);
        }, $rows);
    }
    public function find($id)
    {
        $stmt = $this->pdo->prepare("SELECT * FROM " . $this->tableName . " WHERE id = :id");
        $stmt->execute(["id" => $id]);
        $row = $stmt->fetch();

        if ($row == null) return null;
        else return $this->arrayToOrderResult($row);
    }

    public function insert(OrderResult $orderResult)
    {
        $row = $this->orderResultToArray($orderResult);

        $stmt = $this->pdo->prepare("INSERT INTO " . $this->tableName
            . "('id', 'success', 'message', 'insertTimestamp', 'orderData')"
            . "VALUES"
            . "(:id, :success, :message, :insertTimestamp, :orderData)"
        );
        $stmt->execute($row);
    }

    public function insertMany($orderResultList)
    {
        foreach ($orderResultList as $orderResult) {
            $this->insert($orderResult);
        }
    }

    protected function orderResultToArray(OrderResult $orderResult)
    {
        $row = [
            "id" => $orderResult->getId(),
            "success" => $orderResult->success,
            "message" => $orderResult->message,
            "insertTimestamp" => $orderResult->insertTimestamp,
            "orderData" => serialize($orderResult->order)
        ];

        return $row;
    }

    /**
     * @param $array
     * @return OrderResult
     */
    protected function arrayToOrderResult($array)
    {
        $orderResult = new OrderResult(
            $array["success"],
            unserialize($array["orderData"]),
            $array["message"],
            $array["insertTimestamp"]
        );

        return $orderResult;
    }

}